;;; mac-win.el --- support for "Macintosh windows", modeled after pc-win.el

;; Copyright (C) 1999 Andrew Choi.

;; Author: Andrew Choi <choi@cs.hku.hk>

;; This file is part of the Macintosh port of GNU Emacs.

;; The Macintosh port of GNU Emacs is open source software; you can
;; redistribute it and/or modify it under the terms of the GNU General Public
;; License as published by the Free Software Foundation; either version 2,
;; or (at your option) any later version.

;; The Macintosh port of GNU Emacs is distributed in the hope that it will
;; be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

;; ---------------------------------------------------------------------------
;; We want to delay setting frame parameters until the faces are setup

(if (fboundp 'new-fontset)
    (require 'fontset))

(if (fboundp 'new-fontset)
    (create-fontset-from-fontset-spec standard-fontset-spec t))

(defvar default-frame-alist nil)

(modify-frame-parameters (car (frame-list)) default-frame-alist)

(defun msdos-bg-mode (&optional frame)
  (let* ((frame (or frame (selected-frame)))
	 (params (frame-parameters frame))
	 (bg (cdr (assq 'background-color params))))
    (if (member bg '("black" "blue" "darkgray" "green"))
	'dark
      'light)))

(defun msdos-face-setup ()
  (modify-frame-parameters (car (frame-list))  default-frame-alist)

  (modify-frame-parameters (car (frame-list)) 
			   (list (cons 'background-mode
				       (msdos-bg-mode (car (frame-list)) ))
				 (cons 'display-type 'color)))

  (face-set-after-frame-default (car (frame-list)) )

  (set-face-foreground 'bold "MidnightBlue" (car (frame-list)) )
  (set-face-foreground 'italic "DarkSlateGray" (car (frame-list)) )
  (set-face-foreground 'bold-italic "OrangeRed4" (car (frame-list)) )

  (set-face-foreground 'default "black" (car (frame-list)) )
  (set-face-background 'default "white" (car (frame-list)) )

  (set-face-foreground 'modeline "white" (car (frame-list)) )
  (set-face-background 'modeline "black" (car (frame-list)) ))

;; Are we running Mac OS X
(defun mac-os-x ()
  (if (string-match "darwin" system-configuration)
	  t))

;; Replace the frame-initialize in frame.el so that when it is called in startup.el,
;; our terminal frame (which is the same as our opening frame) wont' be replaced.
(if (not (mac-os-x))
	(defun frame-initialize () nil))

;; We have only one font, so...
(add-hook 'before-init-hook 'msdos-face-setup)

;; We create frames as if we were a terminal, but with a twist.
; (defun make-msdos-frame (&optional parameters)
;   (let* ((parms
; 	  (append initial-frame-alist default-frame-alist parameters nil))
; 	 (frame (make-terminal-frame parms)))
;     (modify-frame-parameters frame
; 			     (list (cons 'background-mode
; 					 (msdos-bg-mode frame))
; 				   (cons 'display-type 'color)))
;     frame))

(if (mac-os-x)
	(setq frame-creation-function 'x-create-frame-with-faces)
  (setq frame-creation-function 'make-terminal-frame))

(defun modify-frame-params-after-init ()
  (modify-frame-parameters (car (frame-list)) default-frame-alist))

(add-hook 'after-init-hook 'modify-frame-params-after-init)

;; ---------------------------------------------------------------------------
;; More or less useful imitations of certain X-functions.  A lot of the
;; values returned are questionable, but usually only the form of the
;; returned value matters.  Also, by the way, recall that `ignore' is
;; a useful function for returning 'nil regardless of argument.

;; From src/xfns.c
(defun x-display-color-p (&optional display) 't)
;;(defun x-list-fonts (pattern &optional face frame maximum width)
;;  (if (and (numberp width) (= width 1))
;;      (list "default")
;;    (list "no-such-font")))
;;(defun x-color-defined-p (color) (numberp (msdos-color-translate color)))
(defun x-display-pixel-width (&optional frame) (frame-width frame))
(defun x-display-pixel-height (&optional frame) (frame-height frame))
(defun x-display-planes (&optional frame) 4) ; 3 for background, actually
(defun x-display-color-cells (&optional frame) 16) ; ???
(defun x-server-max-request-size (&optional frame) 1000000) ; ???
(defun x-server-vendor (&optional frame) t "GNU")
(defun x-server-version (&optional frame) '(1 0 0))
(defun x-display-screens (&optional frame) 1)
(defun x-display-mm-height (&optional frame) 200) ; Guess the size of my
(defun x-display-mm-width (&optional frame) 253)  ; monitor, MW...
(defun x-display-backing-store (&optional frame) 'not-useful)
(defun x-display-visual-class (&optional frame) 'static-color)
(fset 'x-display-save-under 'ignore)
(fset 'x-get-resource 'ignore)

;; From lisp/term/x-win.el
(setq x-display-name "pc")
(setq split-window-keep-point t)
(defvar x-colors
  '(
  "snow"
  "ghost white"
  "GhostWhite"
  "white smoke"
  "WhiteSmoke"
  "gainsboro"
  "floral white"
  "FloralWhite"
  "old lace"
  "OldLace"
  "linen"
  "antique white"
  "AntiqueWhite"
  "papaya whip"
  "PapayaWhip"
  "blanched almond"
  "BlanchedAlmond"
  "bisque"
  "peach puff"
  "PeachPuff"
  "navajo white"
  "NavajoWhite"
  "moccasin"
  "cornsilk"
  "ivory"
  "lemon chiffon"
  "LemonChiffon"
  "seashell"
  "honeydew"
  "mint cream"
  "MintCream"
  "azure"
  "alice blue"
  "AliceBlue"
  "lavender"
  "lavender blush"
  "LavenderBlush"
  "misty rose"
  "MistyRose"
  "white"
  "black"
  "dark slate gray"
  "DarkSlateGray"
  "dark slate grey"
  "DarkSlateGrey"
  "dim gray"
  "DimGray"
  "dim grey"
  "DimGrey"
  "slate gray"
  "SlateGray"
  "slate grey"
  "SlateGrey"
  "light slate gray"
  "LightSlateGray"
  "light slate grey"
  "LightSlateGrey"
  "gray"
  "grey"
  "light grey"
  "LightGrey"
  "light gray"
  "LightGray"
  "midnight blue"
  "MidnightBlue"
  "navy"
  "navy blue"
  "NavyBlue"
  "cornflower blue"
  "CornflowerBlue"
  "dark slate blue"
  "DarkSlateBlue"
  "slate blue"
  "SlateBlue"
  "medium slate blue"
  "MediumSlateBlue"
  "light slate blue"
  "LightSlateBlue"
  "medium blue"
  "MediumBlue"
  "royal blue"
  "RoyalBlue"
  "blue"
  "dodger blue"
  "DodgerBlue"
  "deep sky blue"
  "DeepSkyBlue"
  "sky blue"
  "SkyBlue"
  "light sky blue"
  "LightSkyBlue"
  "steel blue"
  "SteelBlue"
  "light steel blue"
  "LightSteelBlue"
  "light blue"
  "LightBlue"
  "powder blue"
  "PowderBlue"
  "pale turquoise"
  "PaleTurquoise"
  "dark turquoise"
  "DarkTurquoise"
  "medium turquoise"
  "MediumTurquoise"
  "turquoise"
  "cyan"
  "light cyan"
  "LightCyan"
  "cadet blue"
  "CadetBlue"
  "medium aquamarine"
  "MediumAquamarine"
  "aquamarine"
  "dark green"
  "DarkGreen"
  "dark olive green"
  "DarkOliveGreen"
  "dark sea green"
  "DarkSeaGreen"
  "sea green"
  "SeaGreen"
  "medium sea green"
  "MediumSeaGreen"
  "light sea green"
  "LightSeaGreen"
  "pale green"
  "PaleGreen"
  "spring green"
  "SpringGreen"
  "lawn green"
  "LawnGreen"
  "green"
  "chartreuse"
  "medium spring green"
  "MediumSpringGreen"
  "green yellow"
  "GreenYellow"
  "lime green"
  "LimeGreen"
  "yellow green"
  "YellowGreen"
  "forest green"
  "ForestGreen"
  "olive drab"
  "OliveDrab"
  "dark khaki"
  "DarkKhaki"
  "khaki"
  "pale goldenrod"
  "PaleGoldenrod"
  "light goldenrod yellow"
  "LightGoldenrodYellow"
  "light yellow"
  "LightYellow"
  "yellow"
  "gold"
  "light goldenrod"
  "LightGoldenrod"
  "goldenrod"
  "dark goldenrod"
  "DarkGoldenrod"
  "rosy brown"
  "RosyBrown"
  "indian red"
  "IndianRed"
  "saddle brown"
  "SaddleBrown"
  "sienna"
  "peru"
  "burlywood"
  "beige"
  "wheat"
  "sandy brown"
  "SandyBrown"
  "tan"
  "chocolate"
  "firebrick"
  "brown"
  "dark salmon"
  "DarkSalmon"
  "salmon"
  "light salmon"
  "LightSalmon"
  "orange"
  "dark orange"
  "DarkOrange"
  "coral"
  "light coral"
  "LightCoral"
  "tomato"
  "orange red"
  "OrangeRed"
  "red"
  "hot pink"
  "HotPink"
  "deep pink"
  "DeepPink"
  "pink"
  "light pink"
  "LightPink"
  "pale violet red"
  "PaleVioletRed"
  "maroon"
  "medium violet red"
  "MediumVioletRed"
  "violet red"
  "VioletRed"
  "magenta"
  "violet"
  "plum"
  "orchid"
  "medium orchid"
  "MediumOrchid"
  "dark orchid"
  "DarkOrchid"
  "dark violet"
  "DarkViolet"
  "blue violet"
  "BlueViolet"
  "purple"
  "medium purple"
  "MediumPurple"
  "thistle"
  "snow1"
  "snow2"
  "snow3"
  "snow4"
  "seashell1"
  "seashell2"
  "seashell3"
  "seashell4"
  "AntiqueWhite1"
  "AntiqueWhite2"
  "AntiqueWhite3"
  "AntiqueWhite4"
  "bisque1"
  "bisque2"
  "bisque3"
  "bisque4"
  "PeachPuff1"
  "PeachPuff2"
  "PeachPuff3"
  "PeachPuff4"
  "NavajoWhite1"
  "NavajoWhite2"
  "NavajoWhite3"
  "NavajoWhite4"
  "LemonChiffon1"
  "LemonChiffon2"
  "LemonChiffon3"
  "LemonChiffon4"
  "cornsilk1"
  "cornsilk2"
  "cornsilk3"
  "cornsilk4"
  "ivory1"
  "ivory2"
  "ivory3"
  "ivory4"
  "honeydew1"
  "honeydew2"
  "honeydew3"
  "honeydew4"
  "LavenderBlush1"
  "LavenderBlush2"
  "LavenderBlush3"
  "LavenderBlush4"
  "MistyRose1"
  "MistyRose2"
  "MistyRose3"
  "MistyRose4"
  "azure1"
  "azure2"
  "azure3"
  "azure4"
  "SlateBlue1"
  "SlateBlue2"
  "SlateBlue3"
  "SlateBlue4"
  "RoyalBlue1"
  "RoyalBlue2"
  "RoyalBlue3"
  "RoyalBlue4"
  "blue1"
  "blue2"
  "blue3"
  "blue4"
  "DodgerBlue1"
  "DodgerBlue2"
  "DodgerBlue3"
  "DodgerBlue4"
  "SteelBlue1"
  "SteelBlue2"
  "SteelBlue3"
  "SteelBlue4"
  "DeepSkyBlue1"
  "DeepSkyBlue2"
  "DeepSkyBlue3"
  "DeepSkyBlue4"
  "SkyBlue1"
  "SkyBlue2"
  "SkyBlue3"
  "SkyBlue4"
  "LightSkyBlue1"
  "LightSkyBlue2"
  "LightSkyBlue3"
  "LightSkyBlue4"
  "SlateGray1"
  "SlateGray2"
  "SlateGray3"
  "SlateGray4"
  "LightSteelBlue1"
  "LightSteelBlue2"
  "LightSteelBlue3"
  "LightSteelBlue4"
  "LightBlue1"
  "LightBlue2"
  "LightBlue3"
  "LightBlue4"
  "LightCyan1"
  "LightCyan2"
  "LightCyan3"
  "LightCyan4"
  "PaleTurquoise1"
  "PaleTurquoise2"
  "PaleTurquoise3"
  "PaleTurquoise4"
  "CadetBlue1"
  "CadetBlue2"
  "CadetBlue3"
  "CadetBlue4"
  "turquoise1"
  "turquoise2"
  "turquoise3"
  "turquoise4"
  "cyan1"
  "cyan2"
  "cyan3"
  "cyan4"
  "DarkSlateGray1"
  "DarkSlateGray2"
  "DarkSlateGray3"
  "DarkSlateGray4"
  "aquamarine1"
  "aquamarine2"
  "aquamarine3"
  "aquamarine4"
  "DarkSeaGreen1"
  "DarkSeaGreen2"
  "DarkSeaGreen3"
  "DarkSeaGreen4"
  "SeaGreen1"
  "SeaGreen2"  
  "SeaGreen3"
  "SeaGreen4"
  "PaleGreen1"
  "PaleGreen2"
  "PaleGreen3"
  "PaleGreen4"
  "SpringGreen1"
  "SpringGreen2"
  "SpringGreen3"
  "SpringGreen4"
  "green1"
  "green2"
  "green3"
  "green4"
  "chartreuse1"
  "chartreuse2"
  "chartreuse3"
  "chartreuse4"
  "OliveDrab1"
  "OliveDrab2"
  "OliveDrab3"
  "OliveDrab4"
  "DarkOliveGreen1"
  "DarkOliveGreen2"
  "DarkOliveGreen3"
  "DarkOliveGreen4"
  "khaki1"
  "khaki2"
  "khaki3"
  "khaki4"
  "LightGoldenrod1"
  "LightGoldenrod2"
  "LightGoldenrod3"
  "LightGoldenrod4"
  "LightYellow1"
  "LightYellow2"
  "LightYellow3"
  "LightYellow4"
  "yellow1"
  "yellow2"
  "yellow3"
  "yellow4"
  "gold1"
  "gold2"
  "gold3"
  "gold4"
  "goldenrod1"
  "goldenrod2"
  "goldenrod3"
  "goldenrod4"
  "DarkGoldenrod1"
  "DarkGoldenrod2"
  "DarkGoldenrod3"
  "DarkGoldenrod4"
  "RosyBrown1"
  "RosyBrown2"
  "RosyBrown3"
  "RosyBrown4"
  "IndianRed1"
  "IndianRed2"
  "IndianRed3"
  "IndianRed4"
  "sienna1"
  "sienna2"
  "sienna3"
  "sienna4"
  "burlywood1"
  "burlywood2"
  "burlywood3"
  "burlywood4"
  "wheat1"
  "wheat2"
  "wheat3"
  "wheat4"
  "tan1"
  "tan2"
  "tan3"
  "tan4"
  "chocolate1"
  "chocolate2"
  "chocolate3"
  "chocolate4"
  "firebrick1"
  "firebrick2"
  "firebrick3"
  "firebrick4"
  "brown1"
  "brown2"
  "brown3"
  "brown4"
  "salmon1"
  "salmon2"
  "salmon3"
  "salmon4"
  "LightSalmon1"
  "LightSalmon2"
  "LightSalmon3"
  "LightSalmon4"
  "orange1"
  "orange2"
  "orange3"
  "orange4"
  "DarkOrange1"
  "DarkOrange2"
  "DarkOrange3"
  "DarkOrange4"
  "coral1"
  "coral2"
  "coral3"
  "coral4"
  "tomato1"
  "tomato2"
  "tomato3"
  "tomato4"
  "OrangeRed1"
  "OrangeRed2"
  "OrangeRed3"
  "OrangeRed4"
  "red1"
  "red2"
  "red3"
  "red4"
  "DeepPink1"
  "DeepPink2"
  "DeepPink3"
  "DeepPink4"
  "HotPink1"
  "HotPink2"
  "HotPink3"
  "HotPink4"
  "pink1"
  "pink2"
  "pink3"
  "pink4"
  "LightPink1"
  "LightPink2"
  "LightPink3"
  "LightPink4"
  "PaleVioletRed1"
  "PaleVioletRed2"
  "PaleVioletRed3"
  "PaleVioletRed4"
  "maroon1"
  "maroon2"
  "maroon3"
  "maroon4"
  "VioletRed1"
  "VioletRed2"
  "VioletRed3"
  "VioletRed4"
  "magenta1"
  "magenta2"
  "magenta3"
  "magenta4"
  "orchid1"
  "orchid2"
  "orchid3"
  "orchid4"
  "plum1"
  "plum2"
  "plum3"
  "plum4"
  "MediumOrchid1"
  "MediumOrchid2"
  "MediumOrchid3"
  "MediumOrchid4"
  "DarkOrchid1"
  "DarkOrchid2"
  "DarkOrchid3"
  "DarkOrchid4"
  "purple1"
  "purple2"
  "purple3"
  "purple4"
  "MediumPurple1"
  "MediumPurple2"
  "MediumPurple3"
  "MediumPurple4"
  "thistle1"
  "thistle2"
  "thistle3"
  "thistle4"
  "gray0"
  "grey0"
  "gray1"
  "grey1"
  "gray2"
  "grey2"
  "gray3"
  "grey3"
  "gray4"
  "grey4"
  "gray5"
  "grey5"
  "gray6"
  "grey6"
  "gray7"
  "grey7"
  "gray8"
  "grey8"
  "gray9"
  "grey9"
  "gray10"
  "grey10"
  "gray11"
  "grey11"
  "gray12"
  "grey12"
  "gray13"
  "grey13"
  "gray14"
  "grey14"
  "gray15"
  "grey15"
  "gray16"
  "grey16"
  "gray17"
  "grey17"
  "gray18"
  "grey18"
  "gray19"
  "grey19"
  "gray20"
  "grey20"
  "gray21"
  "grey21"
  "gray22"
  "grey22"
  "gray23"
  "grey23"
  "gray24"
  "grey24"
  "gray25"
  "grey25"
  "gray26"
  "grey26"
  "gray27"
  "grey27"
  "gray28"
  "grey28"
  "gray29"
  "grey29"
  "gray30"
  "grey30"
  "gray31"
  "grey31"
  "gray32"
  "grey32"
  "gray33"
  "grey33"
  "gray34"
  "grey34"
  "gray35"
  "grey35"
  "gray36"
  "grey36"
  "gray37"
  "grey37"
  "gray38"
  "grey38"
  "gray39"
  "grey39"
  "gray40"
  "grey40"
  "gray41"
  "grey41"
  "gray42"
  "grey42"
  "gray43"
  "grey43"
  "gray44"
  "grey44"
  "gray45"
  "grey45"
  "gray46"
  "grey46"
  "gray47"
  "grey47"
  "gray48"
  "grey48"
  "gray49"
  "grey49"
  "gray50"
  "grey50"
  "gray51"
  "grey51"
  "gray52"
  "grey52"
  "gray53"
  "grey53"
  "gray54"
  "grey54"
  "gray55"
  "grey55"
  "gray56"
  "grey56"
  "gray57"
  "grey57"
  "gray58"
  "grey58"
  "gray59"
  "grey59"
  "gray60"
  "grey60"
  "gray61"
  "grey61"
  "gray62"
  "grey62"
  "gray63"
  "grey63"
  "gray64"
  "grey64"
  "gray65"
  "grey65"
  "gray66"
  "grey66"
  "gray67"
  "grey67"
  "gray68"
  "grey68"
  "gray69"
  "grey69"
  "gray70"
  "grey70"
  "gray71"
  "grey71"
  "gray72"
  "grey72"
  "gray73"
  "grey73"
  "gray74"
  "grey74"
  "gray75"
  "grey75"
  "gray76"
  "grey76"
  "gray77"
  "grey77"
  "gray78"
  "grey78"
  "gray79"
  "grey79"
  "gray80"
  "grey80"
  "gray81"
  "grey81"
  "gray82"
  "grey82"
  "gray83"
  "grey83"
  "gray84"
  "grey84"
  "gray85"
  "grey85"
  "gray86"
  "grey86"
  "gray87"
  "grey87"
  "gray88"
  "grey88"
  "gray89"
  "grey89"
  "gray90"
  "grey90"
  "gray91"
  "grey91"
  "gray92"
  "grey92"
  "gray93"
  "grey93"
  "gray94"
  "grey94"
  "gray95"
  "grey95"
  "gray96"
  "grey96"
  "gray97"
  "grey97"
  "gray98"
  "grey98"
  "gray99"
  "grey99"
  "gray100"
  "grey100"
  "dark grey"
  "DarkGrey"
  "dark gray"
  "DarkGray"
  "dark blue"
  "DarkBlue"
  "dark cyan"
  "DarkCyan"
  "dark magenta"
  "DarkMagenta"
  "dark red"
  "DarkRed"
  "light green"
  "LightGreen")
  "The list of colors available on a PC display under MS-DOS.")

(defun x-defined-colors (&optional frame)
  "Return a list of colors supported for a particular frame.
The argument FRAME specifies which frame to try.
The value may be different for frames on different X displays."
  x-colors)

;; Use Lisp verison of ls instead of calling subprocess on ls (faster, don't need to write ls).
(load "ls-lisp")

;; Set this so that Emacs calls subprocesses with "sh" as shell to expand filenames
;; Note no subprocess for the shell is actually started (see run_mac_command in sysdep.c).
(setq shell-file-name "sh")

;; X Window emulation in macterm.c is not complete enough to start a frame without a 
;; minibuffer properly.  Call this to tell ediff library to use a single frame.
(ediff-toggle-multiframe)

;; Emacs must be told we're using an 8-bit code for file names.  Otherwise file names
;; won't be displayed properly in dired mode, etc.
(setq file-name-coding-system 'latin-1)

;;(debug-on-entry 'msdos-face-setup)

;; Set these interfaces to the Mac clipboard.  These functions are defined in macterm.c
(setq interprogram-paste-function 'mac-paste-function)
(setq interprogram-cut-function 'mac-cut-function)

(defun mac-drag-n-drop (event)
  "Edit the files listed in the drag-n-drop event.
Switch to a buffer editing the last file dropped."
  (interactive "e")
  (save-excursion
    ;; Make sure the drop target has positive co-ords
    ;; before setting the selected frame - otherwise it
    ;; won't work.  <skx@tardis.ed.ac.uk>
    (let* ((window (posn-window (event-start event)))
	   (coords (posn-x-y (event-start event)))
	   (x (car coords))
	   (y (cdr coords)))
      (if (and (> x 0) (> y 0))
	  (set-frame-selected-window nil window))
      (mapcar 'find-file (car (cdr (cdr event)))))
  (raise-frame)
  (recenter)))

(global-set-key [drag-n-drop] 'mac-drag-n-drop)

; Define constant values to be set to mac-keyboard-text-encoding
(defconst kTextEncodingMacRoman 0)
(defconst kTextEncodingISOLatin1 513 "0x201")
(defconst kTextEncodingISOLatin2 514 "0x202")

;; This is almost an exact copy of sendmail-sent-it in sendmail.el except mac-sendmail
;; is called instead of call-process-region (on fakemail) to send the piece of mail.
(eval-after-load
    "sendmail"
  '(defun sendmail-send-it ()
     (require 'mail-utils)
     (let ((errbuf (if mail-interactive
		       (generate-new-buffer " sendmail errors")
		     0))
	   (tembuf (generate-new-buffer " sendmail temp"))
	   (case-fold-search nil)
	   resend-to-addresses
	   delimline
	   fcc-was-found
	   (mailbuf (current-buffer)))
       (unwind-protect
	   (save-excursion
	     (set-buffer tembuf)
	     (erase-buffer)
	     (insert-buffer-substring mailbuf)
	     (goto-char (point-max))
	     ;; require one newline at the end.
	     (or (= (preceding-char) ?\n)
		 (insert ?\n))
	     ;; Change header-delimiter to be what sendmail expects.
	     (goto-char (mail-header-end))
	     (delete-region (point) (progn (end-of-line) (point)))
	     (setq delimline (point-marker))
	     (sendmail-sync-aliases)
	     (if mail-aliases
		 (expand-mail-aliases (point-min) delimline))
	     (goto-char (point-min))
	     ;; Ignore any blank lines in the header
	     (while (and (re-search-forward "\n\n\n*" delimline t)
			 (< (point) delimline))
	       (replace-match "\n"))
	     (goto-char (point-min))
	     (let ((case-fold-search t))
	       (goto-char (point-min))
	       (while (re-search-forward "^Resent-\\(to\\|cc\\|bcc\\):" delimline t)
		 (setq resend-to-addresses
		       (save-restriction
			 (narrow-to-region (point)
					   (save-excursion
					     (forward-line 1)
					     (while (looking-at "^[ \t]")
					       (forward-line 1))
					     (point)))
			 (append (mail-parse-comma-list)
				 resend-to-addresses)))
		 ;; Delete Resent-BCC ourselves
		 (if (save-excursion (beginning-of-line)
				     (looking-at "resent-bcc"))
		     (delete-region (save-excursion (beginning-of-line) (point))
				    (save-excursion (end-of-line) (1+ (point))))))
;;; Apparently this causes a duplicate Sender.
;;;	    ;; If the From is different than current user, insert Sender.
;;;	    (goto-char (point-min))
;;;	    (and (re-search-forward "^From:"  delimline t)
;;;		 (progn
;;;		   (require 'mail-utils)
;;;		   (not (string-equal
;;;			 (mail-strip-quoted-names
;;;			  (save-restriction
;;;			    (narrow-to-region (point-min) delimline)
;;;			    (mail-fetch-field "From")))
;;;			 (user-login-name))))
;;;		 (progn
;;;		   (forward-line 1)
;;;		   (insert "Sender: " (user-login-name) "\n")))
	       ;; Don't send out a blank subject line
	       (goto-char (point-min))
	       (if (re-search-forward "^Subject:\\([ \t]*\n\\)+\\b" delimline t)
		   (replace-match "")
		 ;; This one matches a Subject just before the header delimiter.
		 (if (and (re-search-forward "^Subject:\\([ \t]*\n\\)+" delimline t)
			  (= (match-end 0) delimline))
		     (replace-match "")))
	       ;; Put the "From:" field in unless for some odd reason
	       ;; they put one in themselves.
	       (goto-char (point-min))
	       (if (not (re-search-forward "^From:" delimline t))
		   (let* ((login user-mail-address)
			  (fullname (user-full-name))
			  (quote-fullname nil))
		     (if (string-match "[\200-\377]" fullname)
			 (setq fullname (mail-quote-printable fullname t)
			       quote-fullname t))
		     (cond ((eq mail-from-style 'angles)
			    (insert "From: " fullname)
			    (let ((fullname-start (+ (point-min) 6))
				  (fullname-end (point-marker)))
			      (goto-char fullname-start)
			      ;; Look for a character that cannot appear unquoted
			      ;; according to RFC 822.
			      (if (or (re-search-forward "[^- !#-'*+/-9=?A-Z^-~]"
							 fullname-end 1)
				      quote-fullname)
				  (progn
				    ;; Quote fullname, escaping specials.
				    (goto-char fullname-start)
				    (insert "\"")
				    (while (re-search-forward "[\"\\]"
							      fullname-end 1)
				      (replace-match "\\\\\\&" t))
				    (insert "\""))))
			    (insert " <" login ">\n"))
			   ((eq mail-from-style 'parens)
			    (insert "From: " login " (")
			    (let ((fullname-start (point)))
			      (if quote-fullname
				  (insert "\""))
			      (insert fullname)
			      (if quote-fullname
				  (insert "\""))
			      (let ((fullname-end (point-marker)))
				(goto-char fullname-start)
				;; RFC 822 says \ and nonmatching parentheses
				;; must be escaped in comments.
				;; Escape every instance of ()\ ...
				(while (re-search-forward "[()\\]" fullname-end 1)
				  (replace-match "\\\\\\&" t))
				;; ... then undo escaping of matching parentheses,
				;; including matching nested parentheses.
				(goto-char fullname-start)
				(while (re-search-forward 
					"\\(\\=\\|[^\\]\\(\\\\\\\\\\)*\\)\\\\(\\(\\([^\\]\\|\\\\\\\\\\)*\\)\\\\)"
					fullname-end 1)
				  (replace-match "\\1(\\3)" t)
				  (goto-char fullname-start))))
			    (insert ")\n"))
			   ((null mail-from-style)
			    (insert "From: " login "\n"))
			   ((eq mail-from-style 'system-default)
			    nil)
			   (t (error "Invalid value for `mail-from-style'")))))
	       ;; Insert an extra newline if we need it to work around
	       ;; Sun's bug that swallows newlines.
	       (goto-char (1+ delimline))
	       (if (eval mail-mailer-swallows-blank-line)
		   (newline))
	       ;; Find and handle any FCC fields.
	       (goto-char (point-min))
	       (if (re-search-forward "^FCC:" delimline t)
		   (progn
		     (setq fcc-was-found t)
		     (mail-do-fcc delimline)))
	       (if mail-interactive
		   (save-excursion
		     (set-buffer errbuf)
		     (erase-buffer))))
	     (goto-char (point-min))
	     (if (let ((case-fold-search t))
		   (re-search-forward "^To:\\|^cc:\\|^bcc:\\|^resent-to:\
\\|^resent-cc:\\|^resent-bcc:"
				      delimline t))
		 (let* ((default-directory "/")
			(coding-system-for-write (select-message-coding-system))
			(args 
			 (append (list (point-min) (point-max)
				       (if (boundp 'sendmail-program)
					   sendmail-program
					 "/usr/lib/sendmail")
				       nil errbuf nil "-oi")
				 ;; Always specify who from,
				 ;; since some systems have broken sendmails.
				 ;; unless user has said no.
				 (if (memq mail-from-style '(angles parens nil))
				     (list "-f" user-mail-address))
;;; 			      ;; Don't say "from root" if running under su.
;;; 			      (and (equal (user-real-login-name) "root")
;;; 				   (list "-f" (user-login-name)))
				 (and mail-alias-file
				      (list (concat "-oA" mail-alias-file)))
				 (if mail-interactive
				     ;; These mean "report errors to terminal"
				     ;; and "deliver interactively"
				     '("-oep" "-odi")
				   ;; These mean "report errors by mail"
				   ;; and "deliver in background".
				   '("-oem" "-odb"))
				 ;; Get the addresses from the message
				 ;; unless this is a resend.
				 ;; We must not do that for a resend
				 ;; because we would find the original addresses.
				 ;; For a resend, include the specific addresses.
				 (or resend-to-addresses
				     '("-t"))))
			;; ### call mac-sendmail instead of call-process-region ###
			(exit-value (if (eq window-system 'mac)
					(mac-sendmail)
				      (apply 'call-process-region args))))
		   (or (null exit-value) (zerop exit-value)
		       (error "Sending...failed with exit value %d" exit-value)))
	       (or fcc-was-found
		   (error "No recipients")))
	     (if mail-interactive
		 (save-excursion
		   (set-buffer errbuf)
		   (goto-char (point-min))
		   (while (re-search-forward "\n\n* *" nil t)
		     (replace-match "; "))
		   (if (not (zerop (buffer-size)))
		       (error "Sending...failed to %s"
			      (buffer-substring (point-min) (point-max)))))))
	 (kill-buffer tembuf)
	 (if (bufferp errbuf)
	     (kill-buffer errbuf))))))

(defun mac-sendmail ()
    (save-excursion
      (goto-char (point-min))
      (let
	  ((headerbuf (generate-new-buffer " sendmail header"))
	   (bodybuf (generate-new-buffer " sendmail body"))
	   (mailbuf (current-buffer))
	   (beginningofheader (point-min))
	   (endofbody (point-max))
	   (endofheader (- (re-search-forward "\n\n") 2))
	   to
	   subject
	   cc
	   bcc
	   (temp-body-filename (make-temp-name
				(expand-file-name "sendmail" temporary-file-directory))))
	;; determine the header portion of the message and put it in headerbuf
	(set-buffer headerbuf)
	(insert-buffer-substring mailbuf beginningofheader endofheader)
	;; merge continuation lines
	(goto-char (point-min))
	(while (re-search-forward "\n[ \t]+" (point-max) t)
	  (replace-match " "))
	;; extract to, subject, cc, and bcc fields
	(goto-char (point-min))
	(let ((case-fold-search t))
	  (if (re-search-forward "^Subject: \\(.*\\)$" (point-max) t)
	      (setq subject (match-string 1)))
	  (goto-char (point-min))
	  (if (re-search-forward "^To: \\(.*\\)$" (point-max) t)
	      (setq to (match-string 1)))
	  (goto-char (point-min))
	  (if (re-search-forward "^Cc: \\(.*\\)$" (point-max) t)
	      (setq cc (match-string 1)))
	  (goto-char (point-min))
	  (if (re-search-forward "^Bcc: \\(.*\\)$" (point-max) t)
	      (setq bcc (match-string 1))))
	(kill-buffer headerbuf)
	;; put the body portion of the message in bodybuf
	(set-buffer bodybuf)
	(insert-buffer-substring mailbuf (+ endofheader 2) endofbody)
 	(let ((coding-system-for-write 'raw-text-mac))
 	  (write-region (point-min) (point-max) temp-body-filename))
 	(let ((script
	       (concat
		"set theFileRef to open for access file \""
		(unix-filename-to-mac temp-body-filename) "\"\r"
		"set theText to read theFileRef\r"
		"close access theFileRef\r"
		"tell application \"Eudora Light\"\r"
		"  set newMsg to make new message at end of mailbox \"Out\" of mail folder \"\"\r"
		"  set field \"Subject:\" of newMsg to \"" subject "\"\r"
		"  set field \"To:\" of newMsg to \"" to "\"\r"
		"  set field \"Cc:\" of newMsg to \"" cc "\"\r"
		"  set field \"Bcc:\" of newMsg to \"" bcc "\"\r"
		"  set body of newMsg to theText\r"
		"  set wrap of newMsg to false\r"
		"  with timeout of 15 seconds\r"
		"    queue newMsg\r"
		"    connect with sending without checking\r"
		"  end timeout\r"
		"end tell"
		)))
	  (do-applescript script))
	(delete-file temp-body-filename)
	(kill-buffer bodybuf)))
    0)

(eval-after-load
    "rmail"
  '(defun rmail-insert-inbox-text (files renamep)
     ;; Detect a locked file now, so that we avoid moving mail
     ;; out of the real inbox file.  (That could scare people.)
     (or (memq (file-locked-p buffer-file-name) '(nil t))
	 (error "RMAIL file %s is locked"
		(file-name-nondirectory buffer-file-name)))
     (let (file tofile delete-files movemail popmail got-password password)
       (while files
	 ;; Handle POP mailbox names specially; don't expand as filenames
	 ;; in case the userid contains a directory separator.
	 (setq file (car files))
	 (setq popmail (string-match "^po:" file))
	 (if popmail
	     (setq renamep t)
	   (setq file (file-truename
		       (expand-file-name (substitute-in-file-name file)))))
	 (setq tofile (expand-file-name
		       ;; Generate name to move to from inbox name,
		       ;; in case of multiple inboxes that need moving.
		       (concat ".newmail-" (file-name-nondirectory file))
		       ;; Use the directory of this rmail file
		       ;; because it's a nuisance to use the homedir
		       ;; if that is on a full disk and this rmail
		       ;; file isn't.
		       (file-name-directory
			(expand-file-name buffer-file-name))))
	 ;; Always use movemail to rename the file,
	 ;; since there can be mailboxes in various directories.
	 (setq movemail t)
;;;      ;; If getting from mail spool directory,
;;;      ;; use movemail to move rather than just renaming,
;;;      ;; so as to interlock with the mailer.
;;;      (setq movemail (string= file
;;;			      (file-truename
;;;			       (concat rmail-spool-directory
;;;				       (file-name-nondirectory file)))))
	 (if (and movemail (not popmail))
	     (progn
	       ;; On some systems, /usr/spool/mail/foo is a directory
	       ;; and the actual inbox is /usr/spool/mail/foo/foo.
	       (if (file-directory-p file)
		   (setq file (expand-file-name (user-login-name)
						file)))))
	 (cond (popmail
		(if rmail-pop-password-required
		    (progn (setq got-password (not (rmail-have-password)))
			   (setq password (rmail-get-pop-password))))
		(if (eq system-type 'windows-nt)
		    ;; cannot have "po:" in file name
		    (setq tofile
			  (expand-file-name
			   (concat ".newmail-pop-"
				   (file-name-nondirectory (substring file 3)))
			   (file-name-directory
			    (expand-file-name buffer-file-name)))))
		(message "Getting mail from post office ..."))
	       ((and (file-exists-p tofile)
		     (/= 0 (nth 7 (file-attributes tofile))))
		(message "Getting mail from %s..." tofile))
	       ((and (file-exists-p file)
		     (/= 0 (nth 7 (file-attributes file))))
		(message "Getting mail from %s..." file)))
	 ;; Set TOFILE if have not already done so, and
	 ;; rename or copy the file FILE to TOFILE if and as appropriate.
	 (cond ((not renamep)
		(setq tofile file))
	       ((or (file-exists-p tofile) (and (not popmail)
						(not (file-exists-p file))))
		nil)
	       ((and (not movemail) (not popmail))
		;; Try copying.  If that fails (perhaps no space) and
		;; we're allowed to blow away the inbox, rename instead.
		(if rmail-preserve-inbox
		    (copy-file file tofile nil)
		  (condition-case nil
		      (copy-file file tofile nil)
		    (error
		     ;; Third arg is t so we can replace existing file TOFILE.
		     (rename-file file tofile t))))
		;; Make the real inbox file empty.
		;; Leaving it deleted could cause lossage
		;; because mailers often won't create the file.
		(if (not rmail-preserve-inbox)
		    (condition-case ()
			(write-region (point) (point) file)
		      (file-error nil))))
	       (t
		(let ((errors nil))
		  (unwind-protect
		      (save-excursion
			(setq errors (generate-new-buffer " *rmail loss*"))
			(buffer-disable-undo errors)
			(if (eq window-system 'mac)
			    (mac-movemail file tofile)
			  (let ((args 
				 (append 
				  (list (or rmail-movemail-program
					    (expand-file-name "movemail"
							      exec-directory))
					nil errors nil)
				  (if rmail-preserve-inbox 
				      (list "-p")
				    nil)
				  rmail-movemail-flags
				  (list file tofile)
				  (if password (list password) nil))))
			    (apply 'call-process args)))
			(if (not (buffer-modified-p errors))
			    ;; No output => movemail won
			    nil
			  (set-buffer errors)
			  (subst-char-in-region (point-min) (point-max)
						?\n ?\  )
			  (goto-char (point-max))
			  (skip-chars-backward " \t")
			  (delete-region (point) (point-max))
			  (goto-char (point-min))
			  (if (looking-at "movemail: ")
			      (delete-region (point-min) (match-end 0)))
			  (beep t)
			  (message "movemail: %s"
				   (buffer-substring (point-min)
						     (point-max)))
			  ;; If we just read the password, most likely it is
			  ;; wrong.  Otherwise, see if there is a specific
			  ;; reason to think that the problem is a wrong passwd.
			  (if (or got-password
				  (re-search-forward rmail-pop-password-error
						     nil t))
			      (rmail-set-pop-password nil))
			  (sit-for 3)
			  nil))
		    (if errors (kill-buffer errors))))))
	 ;; At this point, TOFILE contains the name to read:
	 ;; Either the alternate name (if we renamed)
	 ;; or the actual inbox (if not renaming).
	 (if (file-exists-p tofile)
	     (let ((coding-system-for-read
		    (cond ((eq window-system 'mac) 'raw-text-mac)
			  (t 'no-conversion)))
		   size)
	       (goto-char (point-max))
	       (setq size (nth 1 (insert-file-contents tofile)))
	       (goto-char (point-max))
	       (or (= (preceding-char) ?\n)
		   (zerop size)
		   (insert ?\n))
	       (if (not (and rmail-preserve-inbox (string= file tofile)))
		   (setq delete-files (cons tofile delete-files)))))
	 (message "")
	 (setq files (cdr files)))
       delete-files)))

(defun mac-movemail (file tofile)
  (let
      ((script
	(concat
	 "tell application \"Eudora Light\"\r"
	 "  set oldCheckMailSetting to get setting 6\r"
	 "  set setting 6 to \"\"\r"
	 "  with timeout of 30 seconds\r"
	 "    connect with checking without sending\r"
	 "  end timeout\r"
	 "end tell\r"
	 "if (get eof file \"" (unix-filename-to-mac file) "\") equals 0 then\r"
	 "  set theText to \"\"\r"
	 "else\r"
	 "  set inboxFileRef to open for access file \""
	 (unix-filename-to-mac file) "\"\r"
	 "  set theText to read inboxFileRef\r"
	 "  close access inboxFileRef\r"
	 "end if\r"
	 "set tofileFileRef to open for access file \""
	 (unix-filename-to-mac tofile) "\" with write permission\r"
	 "write theText to tofileFileRef\r"
	 "close access tofileFileRef\r"
	 "tell application \"Eudora Light\"\r"
	 "  repeat with i from 1 to (count messages of mailbox \"In\" of mail folder \"\")\r"
	 "    move first message of mailbox \"In\" of mail folder \"\" to end of mailbox \"Trash\" of mail folder \"\"\r"
	 "  end repeat\r"
	 "  set setting 6 to oldCheckMailSetting\r"
	 "end tell"	 
	 )))
    (do-applescript script)))

;;; mac-win.el ends here
